package com.wjholden.kurosuke;

import android.app.Activity;
import android.content.Context;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Bundle;
import android.os.Message;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.Spinner;

public class KurosukeController extends Activity implements SensorEventListener, KurosukeConstants
{
	
	private Button mBtnA;
	private Button mBtnB;
	private Spinner mPlayerSelectSpinner;
	public static SensorManager sm;
	
	private NetworkLogic nl;
	
	private int mPlayerId;
	
	private int up, down, left, right, btnA, btnB;

    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        
        nl = new NetworkLogic();
        nl.start();
        
        sm = (SensorManager) getSystemService(Context.SENSOR_SERVICE);
        
        mBtnA = (Button) findViewById(R.id.Button01);
        mBtnB = (Button) findViewById(R.id.Button02);
        
        mPlayerSelectSpinner = (Spinner) findViewById(R.id.Spinner01);
        ArrayAdapter<CharSequence> adapterCommand = ArrayAdapter.createFromResource(
				this, R.array.playerId, android.R.layout.simple_spinner_item);
		adapterCommand.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
		mPlayerSelectSpinner.setAdapter(adapterCommand);
        
		mPlayerSelectSpinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {

			@Override
			public void onItemSelected(AdapterView<?> arg0, View arg1, int arg2, long arg3) {
				mPlayerId = Integer.parseInt(mPlayerSelectSpinner.getSelectedItem().toString());
				Log.i(TAG_GUI, "Player ID = " + mPlayerId);
			}

			@Override
			public void onNothingSelected(AdapterView<?> arg0) {
				Log.i(TAG_GUI, "Nothing was selected somehow in mPlayerSelectSpinner.");
			}
		});
		
		mBtnA.setOnClickListener(new View.OnClickListener() {
			
			@Override
			public void onClick(View v) {
				btnA = 1;
				sendCommand();
			}
		});
		
		mBtnB.setOnClickListener(new View.OnClickListener() {
			
			@Override
			public void onClick(View v) {
				btnB = 1;
				sendCommand();
			}
		});
    }

	@Override
	public void onAccuracyChanged(Sensor arg0, int arg1) {
		// TODO Auto-generated method stub
		
	}
	
	@Override
	protected void onResume() {
		super.onResume();
		try
		{
			if (!sm.registerListener(this, sm.getDefaultSensor(Sensor.TYPE_ORIENTATION), SensorManager.SENSOR_DELAY_GAME))
				Log.e(TAG_GUI, "Failed to register sensor listener.");
		}
		catch (IllegalArgumentException e)
		{
			Log.e(TAG_GUI, "Illegal Argument Exception: " + e.toString());
		}
		catch (SecurityException e)
		{
			Log.e(TAG_GUI, "Security Exception: " + e.toString());
		}
		catch (RuntimeException e)
		{
			Log.e(TAG_GUI, "Runtime Exception: " + e.toString());
		} 
		catch (Exception e)
		{
			Log.e(TAG_GUI, e.toString());
		}
	}

	@Override
	public void onSensorChanged(SensorEvent event) {
		float[] orientation = event.values;
		
		int oldL = left;
		int oldR = right;
		int oldD = down;
		int oldU = up;
		
		if (orientation[1] >= 20.0f)
		{
			left = 1;
			right = 0;
		}
		else if (-20.0f < orientation[1] && orientation[1] < 20.0f)
		{
			left = right = 0;
		}
		else
		{
			right = 1;
			left = 0;
		}
		
		if (orientation[2] >= 20.0f)
		{
			down = 1;
			up = 0;
		}
		else if (-20.0f < orientation[2] && orientation[2] < 20.0f)
		{
			up = down = 0;
		}
		else
		{
			up = 1;
			down = 0;
		}
		
		if (!(oldL == left && oldR == right && oldD == down && oldU == up))
		{
			sendCommand();
		}
	}
	
	private void sendCommand()
	{
		Message msg = Message.obtain();
		msg.what = NetworkLogic.SEND_KEYS;
		msg.arg1 = mPlayerId;
		Bundle data = new Bundle();
		data.putInt("up", up);
		data.putInt("down", down);
		data.putInt("left", left);
		data.putInt("right", right);
		data.putInt("a", btnA);
		data.putInt("b", btnB);
		msg.setData(data);
		NetworkLogic.handler.sendMessage(msg);
		
		btnA = btnB = 0; // reset buttons after they've been pressed.
	}

	@Override
	protected void onStop() {
		Message msg = Message.obtain();
		msg.what = NetworkLogic.CLOSE;
		NetworkLogic.handler.sendMessage(msg);
		sm.unregisterListener(this);
		super.onDestroy();
	}
	
	@Override
	public boolean onPrepareOptionsMenu(Menu menu) {
		menu.removeGroup(MENU_GROUP_EXTRA_OPTIONS);
		menu.add(MENU_GROUP_EXTRA_OPTIONS, MENU_ITEM_EXIT, Menu.NONE, "Exit");
		return super.onCreateOptionsMenu(menu);
	}

	@Override
	public boolean onMenuItemSelected(int featureId, MenuItem item) {
		switch (item.getItemId())
		{
		case MENU_ITEM_EXIT:
			this.finish();
			break;
		default:
			Log.d(TAG_GUI, "An unrecognized menu item was selected with id = " + item.getItemId());
			break;
		}
		return super.onMenuItemSelected(featureId, item);
	}
}